/***********************************************************************
 * File: sql_quoting.h
 * 	(Extracted from
 *	   "$Id: sql_quoting.h,v 1.2 2007/10/27 17:42:15 Administrator Exp Administrator $")
 *
 * Abstracts:
 *  This C header defines severals for sql_quoting() API.
 *
 * Copyright (C) 2006-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#ifndef _SQL_QUOTING_H_ /* { */
#define _SQL_QUOTING_H_
#ifdef __cplusplus
extern "C"
{
#endif

/* include the system definitions */
#include <string.h>
#include <errno.h>

/* !!WARNING!! If you need to care the windows, define FOR_WINDOWS.
 * (The macro enables EOF code '\z' for escaping control characters.)
 * (for MS's JET users)
 */
#ifdef FOR_WINDOWS /* { for MS-JET users */
#	define SQL_quoting_CONTROL_CHARACTERS "\n\r\z"
#else /* } { for usually RDBMS users */
#	define SQL_quoting_CONTROL_CHARACTERS "\n\r"
#endif /* } */

/* type definition for escaping qualifier */
typedef enum sql_quoting_qualifier_t {
	  SQL_quoting_default			= 0x0
			/* default escaping:
			 * 1) duplicate the quote character (' or ").
			 * 2) do not escape any control characters.
			 * 3) do not escape any pattern match codes.
			 */
	, SQL_quoting_with_backslash		= 0x1
			/* if you need to escape the quote characters
			 * with a backslash code, set this bit.
			 */
	, SQL_quoting_escape_control_codes	= 0x2
			/* if you need to escape the control characters,
			 * set this bit. (for MySQL users)
			 */
	, SQL_quoting_escape_pattern_codes	= 0x4
			/* if you use LIKE operator in SQL, set this
			 * bit
			 */
	, SQL_quoting_enclose_result		= 0x8
			/* if you need the result that is enclosed with
			 * quote characters, set this bit.
			 */
	, SQL_quoting_do_not_escape_quots	= 0x10
			/* If you use the escaped character to static
			 * embeded SQL describes, set this bit.
			 */
	, SQL_quoting_escape_vertical_bar	= 0x20
} SQL_quoting_qualifier_t;

/* predication to check the qualifier argument */
#define is_legal_SQL_quoting_qualifier(SQC) (((			\
	(SQC) == SQL_quoting_default)				\
	|| ((SQC) & ~(SQL_quoting_with_backslash		\
			| SQL_quoting_escape_control_codes	\
			| SQL_quoting_escape_pattern_codes	\
			| SQL_quoting_enclose_result		\
			| SQL_quoting_do_not_escape_quots	\
			| SQL_quoting_escape_vertical_bar	\
			)) == 0)				\
	)

/* predication to check the quote_char argument */
#define is_legal_SQL_quote_character(QCH) (			\
		(QCH) == '\'' || (QCH) == '"'			\
	)

/* predication to check the escape code argument */
#define is_legal_SQL_escape_code(ESCCD,QUALIFIER) (		\
		(ESCCD) == '\0'					\
		|| ((ESCCD) != '%' && (ESCCD) != '_'		\
			&& (ESCCD) != '\'' && (ESCCD) != '"'	\
			&& (((SQL_quoting_escape_vertical_bar	\
					& (QUALIFIER)) == 0	\
				) || (ESCCD) != '|')		\
		)						\
	)
/*
 * this API escapes a string for the SQL literal:
 *
 * arguments:
 *	1) src points to the source string.
 *	2) qualifier indicates the escaping methods.
 *	3) quote_char specifies quote code of `src' literal in SQL.
 *	4) escape_code specifies a code in SQL `ESCAPE' statement.
 *	5) dst points to the destiny buffer. this API outputs escaped
 *	  string toward it. (size of the buffer should be
 *	  "strlen(src) * 2 + 3")
 *	6) dst_sz indicates the size of destiny buffer.
 *
 * returns:
 *	Not NULL (same as dst) means normal end.
 *	NULL occurred several errors. errno indicates the detail:
 *		EINVAL:	  illegal arguments
 *		EMSGSIZE: buffer over run (size of the destiny buffer is
 *			  insufficient)
 */
extern char *sql_quoting(char *src
			, SQL_quoting_qualifier_t qualifier
			, char quote_char
			, char escape_code
			, char *dst
			, size_t dst_sz
			);

#ifdef __cplusplus
}
#endif
#endif /* } _SQL_QUOTING_H_ */
