/***********************************************************************
 * File: test.c
 * 	(Extracted from
 *	   "$Id: test.c,v 1.1 2007/10/30 09:35:30 Administrator Exp Administrator $")
 *
 * Abstracts:
 *  This C source code implements a test program for buffer_checker.c
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include <stdlib.h>
#include <stdio.h>

/* At first, API users must include the following header file. */
#include <buffer_checker.h>

/* Second, API users must define the buffer structure layout. */
typedef struct buffer_t {
#	define NAME_MAX_LEN	50
	char	name[NAME_MAX_LEN + 1];

#	define JP_NAME_MAX_LEN 10
	char	jp_name[JP_NAME_MAX_LEN + 1];

#	define ADDRESS_MAX_LEN	100
	char	address[ADDRESS_MAX_LEN + 1];

#	define EMAIL_MAX_LEN	50
	char	email[EMAIL_MAX_LEN + 1];

	int	somethig_counter;
#	define	SOMETHIG_COUNTER_INITVAL	20041224
} Buffer_t;


/* Third, API users must define the prototypes matched or unmatched
 * callbacks (and field checker callback (if user needs it))
 */
static void *get_int_addr(void	 *buffer
			, ssize_t fld_offset
			, size_t *fldszp
			, void	 *user_cntxt);

static ssize_t check_int_field(size_t	 checker_index
			, void		*buffer
			, size_t	 buffer_sz
			, void		*fldp
			, Buffer_checker_regex_result_t
					 regex_result
			, void		*user_cntxt
			);

static int matched(size_t	 checker_index
		, void		*buffer
		, size_t	 buffer_sz
		, void		*fldp
		, void		*passed
		);

static int unmatched(size_t	 checker_index
		, void		*buffer
		, size_t	 buffer_sz
		, void		*fldp
		, void		*passed
		);


/* Forth, API users must describe the buffer-checker configuration.
 *
 * 1) Buffer_checker_fld() macro is replaced offsetof() and fld_sizeof()
 *   macros.
 * 2) The second member (NULL) is a pointer to function that returns
 *   the field address in the buffer.
 * 3) Buffer_checker_RE() macro is replaced `RE, NULL' C-describe.
 *   And, Buffer_checker_ANY_MATCH_RE macro is replaced `"^.*$", NULL'.
 * 4) The forth and fifth members are callbacks at matched or unmatched.
 */
static Buffer_checker_t defines[] = {
  { Buffer_checker_fld(Buffer_t, name), NULL
	, Buffer_checker_RE("^[A-Za-z\\.]+ [A-Za-z\\. ]+$"
			, matched
			, unmatched
			)
	}

, { Buffer_checker_fld(Buffer_t, jp_name), NULL
	, Buffer_checker_ANY_MATCH_RE(matched)
	    /* Japanese field should be processed by match(),always. */
	}

, { Buffer_checker_fld(Buffer_t, address), NULL
	, Buffer_checker_RE(
			  "^[A-Za-z]+ ([A-Za-z ]*)[0-9]+-[0-9][-0-9]+$"
			, matched
			, unmatched
			)
	}

, { Buffer_checker_fld(Buffer_t, email), NULL
	, Buffer_checker_COND(
		"^([^@\\\\| \"'<>]+)@"
			"([^@\\\\| \"'<>\\.]+)\\.[^@\\\\| \"'<>]+$"
		, check_int_field
		, matched
		, unmatched
		, NULL
		)
	}

, { Buffer_checker_fld(Buffer_t, somethig_counter), get_int_addr
	, Buffer_checker_COND(NULL, check_int_field
			, matched
			, unmatched
			, NULL
			)
	}
};
#define DEFINES_CNT (sizeof(defines) / sizeof(defines[0]))


/* Fifth, API users must implement the callbacks. */
static void *get_int_addr(void	 *buffer
			, ssize_t fld_offset
			, size_t *fldszp
			, void	 *user_cntxt)
{
	fprintf(stderr, "get_int_addr(): fld_offset = %ld.\n"
			"\tbuffer = %p. fldszp = %p (*fldszp = %u). "
			"user_cntxt = %p.\n"
		,				(long)fld_offset	
		,		buffer,	(void *)fldszp,		*fldszp
		,			user_cntxt
		); fflush(stderr);

	return	(void *)((char *)buffer + fld_offset);
}


static ssize_t check_int_field(size_t	 checker_index
			, void		*buffer
			, size_t	 buffer_sz
			, void		*fldp
			, Buffer_checker_regex_result_t
					 regex_result
			, void		*user_cntxt
			)
{
	fprintf(stderr, "check_int_field(): check_index = %lu. "
				"fldp = %p (*((int *)fldp) = %d). "
				"regex_result = %d.\n"
		"\tbuffer = %p. buffer_sz = %lu. user_cntxt = %p.\n"
		,			(unsigned long)checker_index
		,			fldp,		*((int *)fldp)
		,				regex_result
		,	buffer,	(unsigned long)buffer_sz, user_cntxt
		); fflush(stderr);

	return	(ssize_t)((regex_result == BCRR_Matched)? 0:
			  (regex_result == BCRR_Matched)? 1:
							  2
			 );
}


static int matched(size_t	 checker_index
		, void		*buffer
		, size_t	 buffer_sz
		, void		*fldp
		, void		*passed
		)
{	/* This will be call-backed, when the field is matched
	 * with RE
	 */
	unsigned long	p = *((unsigned long *)passed);
	*((unsigned long *)passed) = (p |= (1 << checker_index));
	fprintf(stderr, "matched(): checker_index = %d. passed = %lu.\n"
			"\t[regex = \"%s\"]. "
						"preg = %p.\n"
			"\tfldp = \"%s\". buffer = %p. "
						"buffer_sz = %lu.\n"
			,		checker_index,	(unsigned long)p
			,	defines[checker_index].regex
			,	(void *)(defines[checker_index].preg)
			,	(char *)fldp,	buffer
			,			(unsigned long)buffer_sz
			);
	return	BCR_Continue;
}


static int unmatched(size_t	 checker_index
		, void		*buffer
		, size_t	 buffer_sz
		, void		*fldp
		, void		*passed
		)
{	/* This will be callback-ed, when the field is unmatched */
	fprintf(stderr, "unmatched(): checker_index = %d. "
						"passed = %d.\n"
			"\t[regex = \"%s\"]. "
						"preg = %p.\n"
			"\tfldp = \"%s\". buffer = %p. "
						"buffer_sz = %lu.\n"
			,		checker_index
			,				*(int *)passed
			,	defines[checker_index].regex
			,	(void *)(defines[checker_index].preg)
			,	(char *)fldp,	buffer
			,			(unsigned long)buffer_sz
			);

	return	((checker_index == DEFINES_CNT - 1)
		&& strstr((const char *)fldp, "ababa@") != NULL)?
			BCR_Unmatched:	BCR_Continue;
}


/* At last, API users can invoke the API-s;
 *			init_buffer_checker(), exec_buffer_checker(),
 *			and finish_buffer_checker()
 */
int main(int ac, char **av)
{
	static Buffer_t buffer[] = { /* test data */
		  { "ababa BABAB", ""
			, "Japan Tokyo Shinjuku 1-2-3"
			, "ababa@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* all fields are matched. */
		, { "ababaBABABA", ""
			, "Japan Tokyo Shinjuku 1-2-3"
			, "ababa@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* name is unmatched. */
		, { "0123 45678", ""
			, "Japan Tokyo Shinjuku 1-2-3"
			, "ababa@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* name is unmatched. */
		, { "ababa BABAB", ""
			, "Ginza 12-34-567"
			, "ababa@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* all fields are matched. */
		, { "ababa BABAB", "ACEGI"
			, "1-2-3 Ginza"
			, "ababa@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* address is unmatched. */
		, { "ababa BABAB", "ACEGI"
			, "Tokio Shinjuku 1-2-3"
			, "@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", "UE0`aiu"
			, "Tokio Shinjuku 1-2-3"
			, "ababa@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* all fields are matched. */
		, { "ababa BABAB", ""
			, "Japan Tokyo Setagaya 1-2-3"
			, "a\\baba@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokyo Setagaya 1-2-3"
			, "a|baba@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokyo Setagaya 1-2-3"
			, "a baba@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokyo Setagaya 1-2-3"
			, "a\"baba@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokyo Setagaya 1-2-3"
			, "a'baba@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokyo Setagaya 1-2-3"
			, "a<baba@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokyo Setagaya 1-2-3"
			, "a>baba@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokyo Setagaya 1-2-3"
			, "a-baba@site.mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* all fields are matched. */
		, { "ababa BABAB", ""
			, "Japan Tokio Setagaya 1-2-3"
			, "a-baba@site@mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokio Setagaya 1-2-3"
			, "a-baba@site\\mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokio Setagaya 1-2-3"
			, "a-baba@site|mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokio Setagaya 1-2-3"
			, "a-baba@site mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokio Setagaya 1-2-3"
			, "a-baba@site\"mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokio Setagaya 1-2-3"
			, "a-baba@site'mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokio Setagaya 1-2-3"
			, "a-baba@site<mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", ""
			, "Japan Tokio Setagaya 1-2-3"
			, "a-baba@site>mail.com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched. */
		, { "ababa BABAB", "!\"#$ &'()"
			, "Japan Tokio Setagaya 1-2-3"
			, "ababa@site_mail_com"
			, SOMETHIG_COUNTER_INITVAL
			} /* email is unmatched, and
			   * exec_buffer_checker() returns BCR_Unmatched
			   */
		};
	Buffer_checker_result_t	result;
	size_t	i, executed_cnt = 0;
	unsigned long passed = 0L;
	int	exit_code = 0;

	/* for debugging, dump preg addresses */
	dump_buffer_checker_regex_preg_list(1);
	fprintf(stderr, "\n"); fflush(stderr);

	/* At first, API users must initialize
	 * the field definition describes.
	 */
	if ((result = init_buffer_checker(defines
				, DEFINES_CNT
				, sizeof(Buffer_t)
				)) != BCR_Continue) {
		fprintf(stderr, "init_buffer_checker() returns %d.\n"
			,					result
			);
		exit_code = 1;
	}
	else {	/* process the each buffer... */
		/* for debugging, dump preg addresses */
		dump_buffer_checker_regex_preg_list(1);
		fprintf(stderr, "\n"); fflush(stderr);

		for (i = 0
		; (result == BCR_Continue)
			&& i < sizeof(buffer) / sizeof(buffer[0])
		; ++i, passed = 0L) {
			/* Process a buffer (API users usually receives
			 * or send it);
			 *
			 * `exec_buffer_checker()' API will callback
			 * matched or unmatched functions those are
			 * implemented by API users.
			 */
			executed_cnt	= 0;
			result = exec_buffer_checker(buffer + i
						, sizeof(buffer[0])
						, defines
						, DEFINES_CNT
						, &passed
						, &executed_cnt
						);

			/* print the result and outputs from the API */
			fprintf(stderr, "main(): i = %d. result = %d. "
					"passed = 0x%lx. "
					"executed_cnt = %d.\n\n"
					, 	i,		result
					,	passed
					,		executed_cnt
					);
		}
	}

	/* At last, API users must release the resources. */
	if ((result = finish_buffer_checker(defines, DEFINES_CNT))
						!= BCR_Continue) {
		fprintf(stderr, "main(): finish_buffer_checker() "
				"returns %d.\n"
				,	result
				);
		exit(3); return	3; /* failed finish_buffer_checker() */
	}

	/* for debugging, dump preg addresses */
	dump_buffer_checker_regex_preg_list(1);
	fprintf(stderr, "\n"); fflush(stderr);

	exit(0); return	0; /* Normal Exit */
}
